// PackFile.h: interface for the CPackFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(PACKFILE__INCLUDED_)
#define PACKFILE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BaseObject.h"
#include "stdio.h"

namespace FileSystem
{
//!  Pak Header struct. 
/*!
	This struct its the definition of the a pack file header.
	\ingroup Base
	\sa CPakFile
*/
struct TPakHeader
{
	//!file makr
	char mark[25];

	//!number of files
	DWORD numberOfFiles;

	//!block size
	DWORD blocksize;

	//!version
	DWORD version;

	//!subversion
	DWORD subversion;

	//!extra data
	DWORD extra;
};

//!  File info struct. 
/*!
	This struct its a file info struct.
	\sa CPakFile
	\ingroup Base
*/
struct TFileInfo
{
	//!original file name
	char originalFileName[255];

	//!file name
	char fileName[255];
	
	//!uncompressed size
	unsigned long int uncompressSize;

	//!compressed size
	unsigned long int compressSize;

	//!offset in the file
	unsigned long int offset;
};


//!  File info Element struct. 
/*!
	This struct its a file info element struct.
	\sa CPakFile
	\ingroup Base
*/
struct TFileInfoElement
{
	//!a TFileInfo struct
	TFileInfo info;

	//!next element
	TFileInfoElement* next;

	//!prior element
	TFileInfoElement* prior;
};



//!  PackFile class. 
/*!
	This class it's used to handle packfiles.
	\sa CDataManager
	\sa \ref tools
	\ingroup Base
*/
class CPackFile : public CBaseObject 
{
	public : 				
	//! CBaseObject class it's the parent of CPackFile class
	/*!
		CBaseObject class it's the parent of CPackFile class
	*/
	typedef CBaseObject Inherit;

	//!  Default constructor. 
	/*!
		Called before the object creation.
	*/			
	inline CPackFile(){Inherit();};

	//!  Default destructor. 
	/*!
		Called before the object destruction.
	*/
	virtual ~CPackFile(){};

	
	//!  Finalize an CPakFile object. 
	/*!
		You must call this function for finalize a CPakFile Object.
	*/
	CResult finalize();

	//!  Initialize an CPakFile object. 
	/*!
		This function initialize a CPakFile object, olso its created
		the packfile with the contents of a folder.

		\param directory = Directory to convert to a pack file.
		\param fileName = Pak file name.
		\param blockSize = The size of the block in the pack file.
	*/
	CResult initialize(char* directory, char* fileName,  DWORD blockSize);

	//!  Initialize an CPakFile object. 
	/*!
		This function initialize a CPakFile object, its load the data
		from a existing file.
		
		\param fileName = Pak file name.
	*/
	CResult initialize(char* fileName);
	

	//!  Extract contents from a pack file. 
	/*!
		This function extrac pack file contents into a directory.
		
		\param directory = output directory.
	*/
	CResult extractToDir(char* directory);
	
	inline TFileInfoElement* getFirstFile(){return firstFile;}
	inline TPakHeader* getHeader(){return &header;}
	inline void getFileName(char* sOut){strcpy(sOut,fileName);}
	

private :

	//! pack file header
	/*!
	    pack file header
	*/
	TPakHeader header;

	//! first file in the pack file
	/*!
	    first file in the pack file
	*/
	TFileInfoElement* firstFile;

	//! last file in the pack file
	/*!
	    last file in the pack file
	*/
	TFileInfoElement* lastFile;

	//!	initialize a pack file 
	/*!
	    initialize a pack file 
	*/
	CResult initialize();
	
	//! Make a pack file from a directory
	/*!
	    Make a pack file from a directory
	*/
	CResult makeFromDir(char* directory);

	//! Load a pack file from a dir
	/*!
	    Load a pack file from a dir
	*/
	CResult loadFromFile(char* fileName);

	//! Add files to the file list
	/*!
	    Add files to the file list
	*/
	CResult addFile(char* fileName,char* basedir);

	//! Add directory to the file list
	/*!
	    Add directory to the file list
	*/
	CResult addDirectory(char *directory,char* basedir);

	//! Save the pack file
	/*!
	    Save the pack file
	*/
	CResult savePackFile(char* fileName);

	//! Save the pack file header
	/*!
	    Save the pack file header
	*/
	CResult saveHeader(FILE* outStream);

	//! Load the pack file header
	/*!
	    Load the pack file header
	*/
	CResult loadHeader(FILE* inputStream);

	//! Save the files bodys
	/*!
	    Save the files bodys
	*/
	CResult saveFilesBodys(FILE* outStream);		

	//! Save the file table
	/*!
	    Save the file table
	*/
	CResult saveFileTable(FILE* outStream);

	//! Load the file table
	/*!
	    Load the file table
	*/
	CResult loadFileTable(FILE* inputStream);

	//! file name of the pack file
	/*!
	    file name of the pack file
	*/
	char fileName[MAX_PATH];

protected:
	//!  Add the body of a giving file into an pack file. 
	/*!
		This functino add the body of a file into a pack file,
		you should overwrite this function if you whant to
		mack a custom packer.
		
		\param outStream = a out FILE pointer.
		\param fileInfo = an TFileInfo pointer.
	*/
	virtual CResult saveFileBody(FILE* outStream,TFileInfo* fileInfo);

	//!  Extract a file from a pack file into a directory. 
	/*!
		This functino extract the body of a file in a pack file, to a directory,
		you should overwrite this function if you whant to
		mack a custom unpacker.
		
		\param file = an TFileInfo struct.
		\param directory = output directory.
	*/
	virtual CResult extractFile(TFileInfo* file, char* directory);
};

}
#endif // !defined(PACKFILE__INCLUDED_)
